#!/usr/bin/env python3

from vismach import *
import hal
import sys

# Machine zero as measured from center surface of the rotary c table
machine_zero_x = -1000
machine_zero_y = 1000
machine_zero_z = 2000

for setting in sys.argv[1:]:
    exec(setting)

c = hal.component('xyzc_tangetial-gui')
# axis_x
c.newpin('axis_x', hal.HAL_FLOAT, hal.HAL_IN)
# axis_y
c.newpin('axis_y', hal.HAL_FLOAT, hal.HAL_IN)
# head vertical slide
c.newpin('axis_z', hal.HAL_FLOAT, hal.HAL_IN)
# rotary_c
c.newpin('rotary_c', hal.HAL_FLOAT, hal.HAL_IN)
# tool offsets
# NOTE: For the circular saw tool offsets are tool_length=diameter/2, tool_diameter=thickness*2
c.newpin('tool_length', hal.HAL_FLOAT, hal.HAL_IN)
c.newpin('tool_diameter', hal.HAL_FLOAT, hal.HAL_IN)
# rot-point offsets distances from machine zero
# to rotation axis ( table C)
c.newpin('rot_axis_x', hal.HAL_FLOAT, hal.HAL_IN)
c.newpin('rot_axis_y', hal.HAL_FLOAT, hal.HAL_IN)
c.ready()

# give endpoint Z values and radii
# resulting cylinder is on the Z axis
class HalToolCylinder(CylinderZ):
    def __init__(self, comp, *args):
        CylinderZ.__init__(self, *args)
        self.comp = c

    def coords(self):
        r = 6   # default if hal pin not set
        # if c.tool_diameter > 0:
        #    r = c.tool_diameter / 2
        # return -c.tool_length, r, 0, r
        # For the circular saw we need to swap length and diameter
        if c.tool_length > 0:
            r = c.tool_length
        return -c.tool_diameter / 2, r, 0, r


class CoordSystem(Collection):
    # creates a visual object for a coordinate system
    def __init__(self, comp, r=4, l=500):
        # set arrow length
        al = l / 10
        self.parts = [
            Color([1, 0, 0], [CylinderX(0, r, l, r)]),
            Color([1, 0, 0], [CylinderX(l, 2 * r, l + al, 1)]),
            Color([0, 1, 0], [CylinderY(0, r, l, r)]),
            Color([0, 1, 0], [CylinderY(l, 2 * r, l + al, 1)]),
            Color([0, 0, 1], [CylinderZ(0, r, l, r)]),
            Color([0, 0, 1], [CylinderZ(l, 2 * r, l + al, 1)]),
        ]


class Point(Collection):
    # creates a visual object for a point
    def __init__(self, comp, r=1, l=20, color=[1, 1, 1]):
        self.parts = [
            Color(color, [CylinderX(-l, r, l, r)]),
            Color(color, [CylinderY(-l, r, l, r)]),
            Color(color, [CylinderZ(-l, r, l, r)]),
        ]


# indicators
# create vismach coordinates to show the origin during construction of the model
vismach_coords = CoordSystem(c, 2, 5000)

# create tcp coordinates (these will rotate with the c-axis rotation)
tcp_coords = CoordSystem(c, 8, 300)
tcp_coords = HalRotate([tcp_coords], c, 'rotary_c', 1, 0, 0, -1)
# create tool coordinates that represent the oriented tool coordinate system
# ()these will rotate with the a- and b-axis rotation)
tool_coords = CoordSystem(c, 2, 100)
# add a visual indicator for the position of the tool-rotation point
tool_coords = Collection([tool_coords, Point(c, 1, 30, [1, 0.8, 0, 1])])
coords = Collection([tcp_coords, tool_coords])
# move the above indicators to by the designated (x,y,z)-home offset
coords = Translate([coords], machine_zero_x, machine_zero_y, machine_zero_z)


# create a visual indicator for the position of the control point
ctrl_pt = Point(c)
# move the control point indicator to the control-position
# ctrl_pt = Translate([ctrl_pt], machine_zero_x, machine_zero_y,  machine_zero_z)
# HalTranslate([tooltip], c, 'tool_length', 0, 0, -1),
# for the circular saw we need to swap tool_length/tool_diameter
# ctrl_pt = HalTranslate([tooltip], c, 'tool_diameter', 0, 0, -1),
ctrl_pt = HalTranslate([ctrl_pt], c, 'tool_length', 0, 1, 0)
ctrl_pt = HalTranslate([ctrl_pt], c, 'tool_diameter', 0, 0, -0.5)


# create a visual for the position of the rotational axis of the rotary C table
# as set by the user with the "rot-axis-x" and "rot-axis-y" sliders in the axis gui
rot_axis = Color([1, 1, 0.3, 1], [CylinderZ(0, 2, 2000, 2)])
# move the rot_axis indicator by the designated (x,y)-home offset
rot_axis = Translate([rot_axis], machine_zero_x, machine_zero_y, 0)
# now move it to the position set by the user
rot_axis = HalTranslate([rot_axis], c, 'rot_axis_x', 1, 0, 0)
rot_axis = HalTranslate([rot_axis], c, 'rot_axis_y', 0, 1, 0)
# /indicators

# tool-side
tooltip = Capture()
tool_cylinder = Collection(
    [
        HalToolCylinder(c),
        # this indicates the spindle nose when no tool-offset is active
        CylinderZ(-0.1, 0, 0.1, 10),
    ]
)
tool = Collection(
    [
        # HalTranslate([tooltip], c, 'tool_length', 0, 0, -1),
        # for the circular saw we need to _translate along the y
        HalTranslate([tooltip], c, 'tool_length', 0, 1, 0),
        tool_cylinder,
    ]
)
tool = Color([1, 0, 1, 0], [tool])
# create the spindle housing that contains the spindle
spindle_housing = Collection(
    [
        # spindle
        Color([0.7, 0.7, 0, 1], [CylinderZ(0, 40, 50, 60)]),
        Color([0.7, 0.7, 0, 1], [CylinderZ(50, 60, 300, 60)]),
    ]
)
spindle_assembly = Collection([tool, spindle_housing, ctrl_pt])

# rotate spindle assembly so the spindle rotational axis is along the y
spindle_assembly = Rotate([spindle_assembly], -90, 1, 0, 0)


# add a block for the y-axis to the spindle_assembly
slide_yz = Color([0.6, 0.8, 0.3, 0], [Box(-250, 200, -250, 250, 3000, 250)])
spindle_yz = Collection([spindle_assembly, slide_yz])
# spindle head and y-slide move with y
spindle_yz = HalTranslate([spindle_yz], c, 'axis_y', 0, 1, 0)
# spindle head and y-slide move with z
spindle_yz = HalTranslate([spindle_yz], c, 'axis_z', 0, 0, 1)
# move the spindle_yz to it's designated home position
spindle_yz = Translate(
    [spindle_yz], machine_zero_x, machine_zero_y, machine_zero_z
)
# /tool-side

# work-side
work = Capture()
# create a simple cube
work_piece = BoxCenteredXY(600, 600, 600)

# create rotary_table_c
rotary_table_c = Color([0.1, 0.7, 0.9, 0], [CylinderZ(0, 920, -800, 920)])
# create 'slots' in x and y on the rotary table
slot_x_w = Color([1, 1, 1, 1], [BoxCenteredXY(1840, 10, 0.9)])
slot_y_w = Color([1, 1, 1, 1], [BoxCenteredXY(10, 1840, 0.9)])
# create indicator of at positive end of x slot
slot_pocket = Color([1, 1, 1, 1], [CylinderZ(0, 20, 1, 20)])
slot_pocket = Translate([slot_pocket], 900, 0, 0)
rotary_table_c = Collection(
    [
        work,
        work_piece,
        rotary_table_c,
        slot_x_w,
        slot_y_w,
        slot_pocket,
    ]
)
# create HAL-link for c-axis rotational joint"
rotary_table_c = HalRotate([rotary_table_c], c, 'rotary_c', 1, 0, 0, -1)


table = Collection(
    [
        rotary_table_c,
        rot_axis,
        coords,
        # block that carries the rotary c
        Color(
            [0.5, 0.5, 0.5, 0], [Box(-1500, -1000, -1300, 1500, 1000, -800)]
        ),
    ]
)
table = HalTranslate([table], c, 'axis_x', -1, 0, 0)
# /work-side

# base
base = Collection(
    [
        # base
        Box(-3500, -1000, -1500, 3500, 3000, -1300),
        # column back
        Box(-2000, 1800, -1500, 2000, 4000, 3500),
    ]
)
base = Color([0.2, 0.2, 0.2, 1], [base])
# /base

model = Collection(
    [
        table,
        spindle_yz,
        base,
    ]
)

main(model, tooltip, work, size=4000, lat=-60, lon=25)
